import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
const _excluded = ["tonal", "color", "version", "elevation", "focus", "selected", "button", "href", "disabled", "onFocus", "onBlur", "InteractionProps", "Component", "className", "children"];
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
import React from 'react';
import { is } from '@onesy/utils';
import { classNames, style as styleMethod, useOnesyTheme } from '@onesy/style-react';
import SurfaceElement from '../Surface';
import InteractionElement from '../Interaction';
import LineElement from '../Line';
import { staticClassName } from '../utils';
const useStyle = styleMethod(theme => ({
  root: {
    position: 'relative',
    borderRadius: `${theme.shape.radius.unit * 3}px`,
    maxWidth: '340px',
    overflow: 'hidden',
    transition: theme.methods.transitions.make('border-radius'),
    // reset
    textDecoration: 'none'
  },
  button: {
    cursor: 'pointer',
    userSelect: 'none',
    '&:active': {
      borderRadius: `${theme.shape.radius.unit * 6}px`
    }
  },
  href: {
    '&:active': {
      borderRadius: `${theme.shape.radius.unit * 6}px`
    }
  },
  disabled: {
    cursor: 'pointer',
    pointerEvents: 'none',
    opacity: theme.palette.visual_contrast.default.opacity.disabled
  }
}), {
  name: 'onesy-Card'
});
const Card = /*#__PURE__*/React.forwardRef((props_, ref) => {
  const theme = useOnesyTheme();
  const props = React.useMemo(() => _objectSpread(_objectSpread(_objectSpread({}, theme?.ui?.elements?.all?.props?.default), theme?.ui?.elements?.onesyCard?.props?.default), props_), [props_]);
  const Line = React.useMemo(() => theme?.elements?.Line || LineElement, [theme]);
  const Surface = React.useMemo(() => theme?.elements?.Surface || SurfaceElement, [theme]);
  const Interaction = React.useMemo(() => theme?.elements?.Interaction || InteractionElement, [theme]);
  const {
      tonal = true,
      color: color_ = 'primary',
      version = 'filled',
      elevation: elevation_ = 0,
      focus: focus_,
      selected,
      button,
      href,
      disabled,
      onFocus: onFocus_,
      onBlur: onBlur_,
      InteractionProps,
      Component = props.href ? 'a' : 'div',
      className,
      children
    } = props,
    other = _objectWithoutProperties(props, _excluded);
  const {
    classes
  } = useStyle();
  const [focus, setFocus] = React.useState(focus_ !== undefined ? focus_ : false);
  let color = color_;
  let elevation = elevation_;
  if (disabled) {
    color = 'neutral';
    elevation = 0;
  }
  const onFocus = React.useCallback(event => {
    if (focus_ === undefined && !disabled) {
      setFocus(true);
      if (is('function', onFocus_)) onFocus_(event);
    }
  }, [focus_, disabled, onFocus_]);
  const onBlur = React.useCallback(event => {
    if (focus_ === undefined && !disabled) {
      setFocus(false);
      if (is('function', onBlur_)) onBlur_(event);
    }
  }, [focus_, disabled, onBlur_]);
  return /*#__PURE__*/React.createElement(Surface, _extends({
    ref: ref,
    gap: 0,
    tabIndex: (href || button) && !disabled ? 0 : undefined,
    color: color,
    tonal: tonal,
    version: version,
    elevation: elevation,
    href: href,
    onFocus: onFocus,
    onBlur: onBlur,
    disabled: disabled,
    AdditionalProps: {
      Component
    },
    Component: Line,
    className: classNames([staticClassName('Card', theme) && [`onesy-Card-root`, focus && `onesy-Card-focus`, disabled && `onesy-Card-disabled`], className, classes.root, href && classes.href, button && classes.button, disabled && [classes.disabled, classes[`disabled_version_${version}`]]])
  }, other), (href || button) && /*#__PURE__*/React.createElement(Interaction, _extends({
    border: false,
    pulse: focus,
    selected: selected
  }, InteractionProps)), React.Children.toArray(children).map(item => (/*#__PURE__*/React.cloneElement(item, {
    tonal: item.props.tonal !== undefined ? item.props.tonal : tonal,
    version: item.props.version !== undefined ? item.props.version : version,
    elevation: item.props.elevation !== undefined ? item.props.elevation : elevation,
    color: item.props.color !== undefined ? item.props.color : color
  }))));
});
Card.displayName = 'onesy-Card';
export default Card;