import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
const _excluded = ["tonal", "color", "version", "renderValue", "icon", "onStart", "onPause", "onStop", "onResume", "Icon", "IconStart", "IconPause", "IconStop", "TreeProps", "TooltipProps", "RoundProgressProps", "NumericTextFieldProps", "IconButtonProps", "LinearProgressProps", "IconProps", "Component", "className"];
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
import React from 'react';
import { clamp, getLeadingZerosNumber, is } from '@onesy/utils';
import { OnesyDate, duration } from '@onesy/date';
import { classNames, style as styleMethod, useOnesyTheme } from '@onesy/style-react';
import IconMaterialAvTimer from '@onesy/icons-material-rounded-react/IconMaterialAvTimerW100';
import IconMaterialPlayArrow from '@onesy/icons-material-rounded-react/IconMaterialPlayArrowW100';
import IconMaterialPause from '@onesy/icons-material-rounded-react/IconMaterialPauseW100';
import IconMaterialStop from '@onesy/icons-material-rounded-react/IconMaterialStopW100';
import FadeElement from '../Fade';
import TypeElement from '../Type';
import TooltipElement from '../Tooltip';
import SurfaceElement from '../Surface';
import NumericTextFieldElement from '../NumericTextField';
import RoundProgressElement from '../RoundProgress';
import LinearProgressElement from '../LinearProgress';
import LineElement from '../Line';
import IconButtonElement from '../IconButton';
import { staticClassName } from '../utils';
const useStyle = styleMethod(theme => ({
  root: {
    minWidth: '314px',
    padding: `${theme.methods.space.value(3, 'px')} ${theme.methods.space.value(5, 'px')}`,
    borderRadius: theme.methods.shape.radius.value('rg', 'px')
  },
  roundWrapper: {
    position: 'relative',
    width: '240px',
    height: '240px'
  },
  linearProgress: {
    borderRadius: theme.methods.shape.radius.value(40, 'px'),
    '&.onesy-LinearProgress-root': {
      height: '6px'
    },
    '& .onesy-LinearProgress-line, & .onesy-LinearProgress-buffer': {
      borderRadius: theme.methods.shape.radius.value(40, 'px')
    }
  },
  roundProgress: {
    '&.onesy-RoundedProgress-root': {
      position: 'absolute',
      inset: '0'
    },
    '& .onesy-RoundedProgress-svg': {
      width: '100%',
      height: '100%',
      margin: '0'
    },
    '& .onesy-RoundedProgress-path-background': {
      stroke: 'currentColor',
      opacity: '0.24'
    },
    '& .onesy-RoundedProgress-path, & .onesy-RoundedProgress-path-background': {
      strokeLinecap: 'round'
    }
  },
  meta: {
    maxWidth: '114px',
    textAlign: 'center'
  },
  numericTextField: {
    width: '70px',
    '& .onesy-TextField-input-wrapper': {
      paddingInline: '0px',
      paddingBlock: theme.methods.space.value(1, 'px'),
      height: 'auto'
    },
    '&.onesy-TextField-value .onesy-TextField-label, &.onesy-TextField-focus .onesy-TextField-label': {
      insetInlineStart: '60%',
      transform: `translate(${theme.direction === 'ltr' ? '-' : ''}50%, -16px) scale(0.667)`
    },
    '& .onesy-TextField-input': _objectSpread(_objectSpread({}, theme.typography.values.h2), {}, {
      textAlign: 'center'
    }),
    '& .onesy-TextField-border': {
      boxShadow: 'none'
    }
  }
}), {
  name: 'onesy-Countdown'
});
const Countdown = /*#__PURE__*/React.forwardRef((props_, ref) => {
  const theme = useOnesyTheme();
  const l = theme.l;
  const props = React.useMemo(() => _objectSpread(_objectSpread(_objectSpread({}, theme?.ui?.elements?.all?.props?.default), theme?.ui?.elements?.onesyCountdown?.props?.default), props_), [props_]);
  const Line = React.useMemo(() => theme?.elements?.Line || LineElement, [theme]);
  const Fade = React.useMemo(() => theme?.elements?.Fade || FadeElement, [theme]);
  const Type = React.useMemo(() => theme?.elements?.Type || TypeElement, [theme]);
  const Tooltip = React.useMemo(() => theme?.elements?.Tooltip || TooltipElement, [theme]);
  const Surface = React.useMemo(() => theme?.elements?.Surface || SurfaceElement, [theme]);
  const NumericTextField = React.useMemo(() => theme?.elements?.NumericTextField || NumericTextFieldElement, [theme]);
  const RoundProgress = React.useMemo(() => theme?.elements?.RoundProgress || RoundProgressElement, [theme]);
  const LinearProgress = React.useMemo(() => theme?.elements?.LinearProgress || LinearProgressElement, [theme]);
  const IconButton = React.useMemo(() => theme?.elements?.IconButton || IconButtonElement, [theme]);
  const {
      tonal = true,
      color = 'primary',
      version = 'linear',
      renderValue,
      icon = true,
      onStart: onStart_,
      onPause: onPause_,
      onStop: onStop_,
      onResume: onResume_,
      Icon: Icon_ = IconMaterialAvTimer,
      IconStart = IconMaterialPlayArrow,
      IconPause = IconMaterialPause,
      IconStop = IconMaterialStop,
      TreeProps: TreeProps_,
      TooltipProps: TooltipProps_,
      RoundProgressProps: RoundProgressProps_,
      NumericTextFieldProps: NumericTextFieldProps_,
      IconButtonProps: IconButtonProps_,
      LinearProgressProps: LinearProgressProps_,
      IconProps: IconProps_,
      Component = 'div',
      className
    } = props,
    other = _objectWithoutProperties(props, _excluded);
  const {
    classes
  } = useStyle();
  const [status, setStatus] = React.useState('initial');
  const [value, setValue] = React.useState(0);
  const [values, setValues] = React.useState({
    hours: 0,
    minutes: 0,
    seconds: 0
  });
  const refs = {
    start: React.useRef(0),
    valuePaused: React.useRef(0),
    value: React.useRef(undefined),
    total: React.useRef(undefined),
    values: React.useRef(undefined),
    animationFrame: React.useRef(undefined)
  };
  refs.value.current = value;
  refs.values.current = values;
  const updateValues = (property, valueItem) => {
    setValues(values_ => _objectSpread(_objectSpread({}, values_), {}, {
      [property]: valueItem
    }));
  };
  const clear = () => {
    cancelAnimationFrame(refs.animationFrame.current);
  };
  React.useEffect(() => {
    return () => {
      // Clean up
      clear();
    };
  }, []);
  const update = () => {
    if (refs.value.current <= 0) {
      // Wait for linear, round progress
      // transitions to end
      setTimeout(() => {
        setStatus('initial');
      }, 1400);
      return clear();
    }
    setValue(refs.valuePaused.current - (OnesyDate.milliseconds - refs.start.current));
    refs.animationFrame.current = requestAnimationFrame(update);
  };
  const onStart = React.useCallback(event => {
    refs.total.current = refs.valuePaused.current = (refs.values.current.hours || 0) * 60 ** 2 * 1e3 + (refs.values.current.minutes || 0) * 60 ** 1 * 1e3 + (refs.values.current.seconds || 0) * 1e3;
    setValue(refs.valuePaused.current);
    setStatus('running');
    setTimeout(() => {
      refs.start.current = OnesyDate.milliseconds;
      refs.animationFrame.current = requestAnimationFrame(update);
    }, 14);
    if (is('function', onStart_)) onStart_(event);
  }, []);
  const onPause = React.useCallback(event => {
    clear();

    // Remember previous value
    refs.valuePaused.current = refs.value.current;
    setStatus('paused');
    if (is('function', onPause_)) onPause_(event);
  }, []);
  const onStop = React.useCallback(event => {
    clear();
    refs.start.current = 0;
    refs.valuePaused.current = 0;
    refs.value.current = 0;
    setStatus('initial');
    setValue(0);
    if (is('function', onStop_)) onStop_(event);
  }, []);
  const onResume = React.useCallback(event => {
    // Update start, valuePaused value
    refs.start.current = OnesyDate.milliseconds;

    // ~60+ fps
    refs.animationFrame.current = requestAnimationFrame(update);
    setStatus('running');
    if (is('function', onResume_)) onResume_(event);
  }, []);
  const IconProps = _objectSpread({}, IconProps_);
  const TooltipProps = _objectSpread({
    portal: true,
    interactive: false
  }, TooltipProps_);
  const NumericTextFieldProps = _objectSpread({
    tonal,
    color: 'inherit',
    size: 'small',
    version: 'text',
    increment: false,
    decrement: false
  }, NumericTextFieldProps_);
  const RoundProgressProps = _objectSpread({
    tonal,
    color,
    rounded: true,
    thickness: 1
  }, RoundProgressProps_);
  const LinearProgressProps = _objectSpread({
    tonal,
    color
  }, LinearProgressProps_);
  const IconButtonProps = _objectSpread({
    tonal,
    color: 'inherit',
    version: 'text'
  }, IconButtonProps_);
  const valueFormat = valueNew_ => {
    let valueNew = '';
    const valueDuration = duration(valueNew_, undefined, true, undefined, ['hour', 'minute', 'second']);
    valueNew += `${getLeadingZerosNumber(valueDuration.hour || 0)}:`;
    valueNew += `${getLeadingZerosNumber(valueDuration.minute || 0)}:`;
    valueNew += `${getLeadingZerosNumber(valueDuration.second || 0)}`;
    return valueNew;
  };
  const value_ = status === 'initial' ? '00:00:00' : valueFormat(value);
  return /*#__PURE__*/React.createElement(Surface, _extends({
    ref: ref,
    tonal: tonal,
    color: color,
    gap: 1,
    direction: "column",
    align: "center",
    Component: Line,
    AdditionalProps: {
      Component
    },
    className: classNames([staticClassName('Countdown', theme) && ['onesy-Countdown-root'], className, classes.root])
  }, other), icon && /*#__PURE__*/React.createElement(Icon_, _extends({
    size: "medium"
  }, IconProps, {
    style: _objectSpread({
      marginBottom: 4
    }, IconProps?.style)
  })), status === 'initial' && /*#__PURE__*/React.createElement(Fade, {
    in: true,
    add: true
  }, /*#__PURE__*/React.createElement(Line, {
    gap: 1,
    direction: "row",
    align: "flex-end",
    style: {
      marginTop: 12,
      marginBottom: -8
    }
  }, /*#__PURE__*/React.createElement(NumericTextField, _extends({
    name: l('Hours'),
    min: 0,
    max: 23,
    value: values.hours,
    onChange: valueNew => updateValues('hours', !valueNew ? 0 : valueNew)
  }, NumericTextFieldProps, {
    className: classNames([staticClassName('Countdown', theme) && ['onesy-Countdown-numeric-text-field'], NumericTextFieldProps?.className, classes.numericTextField])
  })), /*#__PURE__*/React.createElement(NumericTextField, _extends({
    name: l('Minutes'),
    min: 0,
    max: 59,
    value: values.minutes,
    onChange: valueNew => updateValues('minutes', !valueNew ? 0 : valueNew)
  }, NumericTextFieldProps, {
    className: classNames([staticClassName('Countdown', theme) && ['onesy-Countdown-numeric-text-field'], NumericTextFieldProps?.className, classes.numericTextField])
  })), /*#__PURE__*/React.createElement(NumericTextField, _extends({
    name: l('Seconds'),
    min: 0,
    max: 59,
    value: values.seconds,
    onChange: valueNew => updateValues('seconds', !valueNew ? 0 : valueNew)
  }, NumericTextFieldProps, {
    className: classNames([staticClassName('Countdown', theme) && ['onesy-Countdown-numeric-text-field'], NumericTextFieldProps?.className, classes.numericTextField])
  })))), status !== 'initial' && ['regular', 'linear'].includes(version) && /*#__PURE__*/React.createElement(Fade, {
    in: true,
    add: true
  }, /*#__PURE__*/React.createElement(Line, {
    gap: 1,
    direction: "column",
    align: "center",
    justify: "center",
    style: {
      width: '100%'
    }
  }, is('function', renderValue) ? renderValue(value_) : /*#__PURE__*/React.createElement(Type, {
    version: "h1"
  }, value_), version === 'linear' && /*#__PURE__*/React.createElement(LinearProgress, _extends({
    version: "determinate",
    value: clamp(Math.round(value / 1000 / (refs.total.current / 1000) * 100), 0, 100),
    reverse: true,
    style: {
      margin: '4px 0 8px'
    }
  }, LinearProgressProps, {
    className: classNames([staticClassName('Countdown', theme) && ['onesy-Countdown-linear-progress'], LinearProgressProps?.className, classes.linearProgress])
  })))), status !== 'initial' && ['round'].includes(version) && /*#__PURE__*/React.createElement(Fade, {
    in: true,
    add: true
  }, /*#__PURE__*/React.createElement(Line, {
    gap: 0.5,
    direction: "column",
    align: "center",
    justify: "center",
    className: classNames([staticClassName('Countdown', theme) && ['onesy-Countdown-round-wrapper'], classes.roundWrapper])
  }, /*#__PURE__*/React.createElement(RoundProgress, _extends({
    version: "determinate",
    value: clamp(Math.round(value / 1000 / (refs.total.current / 1000) * 100), 0, 100)
  }, RoundProgressProps, {
    className: classNames([staticClassName('Countdown', theme) && ['onesy-Countdown-round-progress'], RoundProgressProps?.className, classes.roundProgress])
  })), is('function', renderValue) ? renderValue(value_) : /*#__PURE__*/React.createElement(Type, {
    version: "h1"
  }, value_), /*#__PURE__*/React.createElement(Type, {
    version: "b3",
    className: classNames([staticClassName('Countdown', theme) && ['onesy-Countdown-meta'], classes.meta])
  }, l('Total'), " ", duration(refs.total.current, false)))), /*#__PURE__*/React.createElement(Line, {
    gap: 1,
    direction: "row",
    align: "center",
    justify: "center"
  }, status === 'initial' && /*#__PURE__*/React.createElement(Fade, {
    in: true,
    add: true
  }, /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement(Tooltip, _extends({
    name: l('Start')
  }, TooltipProps), /*#__PURE__*/React.createElement(IconButton, _extends({
    onClick: onStart,
    disabled: !((values.hours || 0) + (values.minutes || 0) + (values.seconds || 0))
  }, IconButtonProps), /*#__PURE__*/React.createElement(IconStart, null))))), status === 'running' && /*#__PURE__*/React.createElement(Fade, {
    in: true,
    add: true
  }, /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement(Tooltip, _extends({
    name: l('Stop')
  }, TooltipProps), /*#__PURE__*/React.createElement(IconButton, _extends({
    onClick: onStop
  }, IconButtonProps), /*#__PURE__*/React.createElement(IconStop, null))), /*#__PURE__*/React.createElement(Tooltip, _extends({
    name: l('Pause')
  }, TooltipProps), /*#__PURE__*/React.createElement(IconButton, _extends({
    onClick: onPause
  }, IconButtonProps), /*#__PURE__*/React.createElement(IconPause, null))))), status === 'paused' && /*#__PURE__*/React.createElement(Fade, {
    in: true,
    add: true
  }, /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement(Tooltip, _extends({
    name: l('Stop')
  }, TooltipProps), /*#__PURE__*/React.createElement(IconButton, _extends({
    onClick: onStop
  }, IconButtonProps), /*#__PURE__*/React.createElement(IconStop, null))), /*#__PURE__*/React.createElement(Tooltip, _extends({
    name: l('Resume')
  }, TooltipProps), /*#__PURE__*/React.createElement(IconButton, _extends({
    onClick: onResume
  }, IconButtonProps), /*#__PURE__*/React.createElement(IconStart, null)))))));
});
Countdown.displayName = 'onesy-Countdown';
export default Countdown;