import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
const _excluded = ["tonal", "color", "renderValue", "icon", "onStart", "onFlag", "onPause", "onStop", "onResume", "Icon", "IconStart", "IconPause", "IconFlag", "IconStop", "TreeProps", "TooltipProps", "IconButtonProps", "IconProps", "Component", "className"];
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
import React from 'react';
import { is, getLeadingZerosNumber } from '@onesy/utils';
import { OnesyDate, duration } from '@onesy/date';
import { classNames, style as styleMethod, useOnesyTheme } from '@onesy/style-react';
import IconMaterialTimer from '@onesy/icons-material-rounded-react/IconMaterialTimerW100';
import IconMaterialPlayArrow from '@onesy/icons-material-rounded-react/IconMaterialPlayArrowW100';
import IconMaterialPause from '@onesy/icons-material-rounded-react/IconMaterialPauseW100';
import IconMaterialStop from '@onesy/icons-material-rounded-react/IconMaterialStopW100';
import IconMaterialFlag from '@onesy/icons-material-rounded-react/IconMaterialFlagW100';
import FadeElement from '../Fade';
import ExpandElement from '../Expand';
import TypeElement from '../Type';
import TooltipElement from '../Tooltip';
import TreeElement from '../Tree';
import SurfaceElement from '../Surface';
import LineElement from '../Line';
import IconButtonElement from '../IconButton';
import { staticClassName } from '../utils';
const useStyle = styleMethod(theme => ({
  root: {
    minWidth: '300px',
    padding: `${theme.methods.space.value(3, 'px')} ${theme.methods.space.value(5, 'px')}`,
    borderRadius: theme.methods.shape.radius.value('rg', 'px')
  },
  value: {
    marginTop: '8px'
  },
  flags_wrapper: {
    width: '100%',
    paddingTop: theme.methods.space.value(1, 'px')
  },
  flags: {
    width: '100%',
    maxHeight: '170px',
    overflowY: 'auto'
  },
  flag: {
    width: '100%',
    paddingInlineStart: theme.methods.space.value(0.5, 'px')
  },
  actions: {
    marginTop: '8px'
  }
}), {
  name: 'onesy-Timer'
});
const Timer = /*#__PURE__*/React.forwardRef((props_, ref) => {
  const theme = useOnesyTheme();
  const l = theme.l;
  const props = React.useMemo(() => _objectSpread(_objectSpread(_objectSpread({}, theme?.ui?.elements?.all?.props?.default), theme?.ui?.elements?.onesyTimer?.props?.default), props_), [props_]);
  const Line = React.useMemo(() => theme?.elements?.Line || LineElement, [theme]);
  const Fade = React.useMemo(() => theme?.elements?.Fade || FadeElement, [theme]);
  const Type = React.useMemo(() => theme?.elements?.Type || TypeElement, [theme]);
  const Expand = React.useMemo(() => theme?.elements?.Expand || ExpandElement, [theme]);
  const Tooltip = React.useMemo(() => theme?.elements?.Tooltip || TooltipElement, [theme]);
  const Tree = React.useMemo(() => theme?.elements?.Tree || TreeElement, [theme]);
  const Surface = React.useMemo(() => theme?.elements?.Surface || SurfaceElement, [theme]);
  const IconButton = React.useMemo(() => theme?.elements?.IconButton || IconButtonElement, [theme]);
  const {
      tonal = true,
      color = 'primary',
      renderValue,
      icon = true,
      onStart: onStart_,
      onFlag: onFlag_,
      onPause: onPause_,
      onStop: onStop_,
      onResume: onResume_,
      Icon: Icon_ = IconMaterialTimer,
      IconStart = IconMaterialPlayArrow,
      IconPause = IconMaterialPause,
      IconFlag = IconMaterialFlag,
      IconStop = IconMaterialStop,
      TreeProps: TreeProps_,
      TooltipProps: TooltipProps_,
      IconButtonProps: IconButtonProps_,
      IconProps: IconProps_,
      Component = 'div',
      className
    } = props,
    other = _objectWithoutProperties(props, _excluded);
  const {
    classes
  } = useStyle();
  const [status, setStatus] = React.useState('initial');
  const [flags, setFlags] = React.useState([]);
  const [expand, setExpand] = React.useState();
  const [value, setValue] = React.useState(0);
  const refs = {
    root: React.useRef(undefined),
    start: React.useRef(0),
    valuePaused: React.useRef(0),
    value: React.useRef(undefined),
    animationFrame: React.useRef(undefined)
  };
  refs.value.current = value;
  const clear = () => {
    cancelAnimationFrame(refs.animationFrame.current);
  };
  React.useEffect(() => {
    return () => {
      // Clean up
      clear();
    };
  }, []);
  const update = () => {
    setValue(refs.valuePaused.current + (OnesyDate.milliseconds - refs.start.current));
    refs.animationFrame.current = requestAnimationFrame(update);
  };
  const onStart = React.useCallback(event => {
    refs.start.current = OnesyDate.milliseconds;

    // ~60+ fps
    refs.animationFrame.current = requestAnimationFrame(update);
    setStatus('running');
    if (is('function', onStart_)) onStart_(event);
  }, []);
  const onFlag = React.useCallback(event => {
    if (!expand) setExpand(true);
    setFlags(values => [...values, refs.value.current]);
    if (is('function', onFlag_)) onFlag_(event);
  }, [expand]);
  const onPause = React.useCallback(event => {
    clear();

    // Remember previous value
    refs.valuePaused.current = refs.value.current;
    setStatus('paused');
    if (is('function', onPause_)) onPause_(event);
  }, []);
  const onStop = React.useCallback(event => {
    clear();
    setStatus('initial');
    setExpand(false);
    setValue(0);
    refs.start.current = 0;
    refs.valuePaused.current = 0;
    refs.value.current = 0;
    if (is('function', onStop_)) onStop_(event);
  }, []);
  const onResume = React.useCallback(event => {
    // ~60+ fps
    refs.animationFrame.current = requestAnimationFrame(update);

    // Update start, valuePaused value
    refs.start.current = OnesyDate.milliseconds;
    setStatus('running');
    if (is('function', onResume_)) onResume_(event);
  }, []);
  const onExited = React.useCallback(() => {
    setFlags([]);
  }, []);
  const IconProps = _objectSpread({}, IconProps_);
  const TooltipProps = _objectSpread({
    portal: true,
    interactive: false
  }, TooltipProps_);
  const TreeProps = _objectSpread({
    tonal,
    color: 'inherit'
  }, TreeProps_);
  const IconButtonProps = _objectSpread({
    tonal,
    color: 'inherit',
    version: 'text'
  }, IconButtonProps_);
  const valueFormat = valueNew_ => {
    let valueNew = '';
    const valueDuration = duration(valueNew_, undefined, true, undefined, ['hour', 'minute', 'second', 'millisecond']);
    if (valueDuration.hour > 0) valueNew += `${getLeadingZerosNumber(valueDuration.hour)}:`;
    valueNew += `${getLeadingZerosNumber(valueDuration.minute || 0)}:`;
    valueNew += `${getLeadingZerosNumber(valueDuration.second || 0)}.`;
    valueNew += `${getLeadingZerosNumber(Math.floor((valueDuration.millisecond || 0) / 10))}`;
    return valueNew;
  };
  const value_ = status === 'initial' ? '00:00.00' : valueFormat(value);
  return /*#__PURE__*/React.createElement(Surface, _extends({
    ref: item => {
      if (ref) {
        if (is('function', ref)) ref(item);else if (ref?.current) ref.current = item;
      }
      refs.root.current = item;
    },
    tonal: tonal,
    color: color,
    gap: 0,
    direction: "column",
    align: "center",
    Component: Line,
    AdditionalProps: {
      Component
    },
    className: classNames([staticClassName('Timer', theme) && ['onesy-Timer-root'], className, classes.root])
  }, other), icon && /*#__PURE__*/React.createElement(Icon_, _extends({
    size: "medium"
  }, IconProps, {
    style: _objectSpread({
      marginBottom: 4
    }, IconProps?.style)
  })), is('function', renderValue) ? renderValue(value_) : /*#__PURE__*/React.createElement(Type, {
    version: "h1",
    className: classNames([staticClassName('Timer', theme) && ['onesy-Timer-value'], classes.value])
  }, value_), /*#__PURE__*/React.createElement(Expand, {
    in: expand,
    parent: refs.root.current,
    onExited: onExited,
    className: classNames([staticClassName('Timer', theme) && ['onesy-Timer-flags-wrapper'], classes.flags_wrapper])
  }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement(Surface, {
    tonal: tonal,
    color: color
  }, _ref => {
    let {
      palette
    } = _ref;
    return /*#__PURE__*/React.createElement(Tree, _extends({
      openDefault: true,
      middle: l('Flags'),
      indicator: true,
      indicatorPosition: "end",
      parent: refs.root.current,
      TreeProps: {
        className: classNames([staticClassName('Timer', theme) && ['onesy-Timer-flags'], classes.flags])
      }
    }, TreeProps), flags.map((item, index) => /*#__PURE__*/React.createElement(Tree, _extends({
      key: index,
      icon: /*#__PURE__*/React.createElement(IconFlag, {
        size: "small",
        color: palette[40]
      }),
      middle: /*#__PURE__*/React.createElement(Line, {
        direction: "row",
        align: "center",
        justify: "space-between",
        className: classNames([classes.flag])
      }, /*#__PURE__*/React.createElement(Type, {
        version: "b2"
      }, getLeadingZerosNumber(index + 1)), /*#__PURE__*/React.createElement(Type, {
        version: "b2",
        style: {
          opacity: '0.7'
        }
      }, "+", valueFormat(item - (flags[index - 1] || 0))), /*#__PURE__*/React.createElement(Type, {
        version: "b2"
      }, valueFormat(item))),
      noPadding: true,
      indicator: true,
      indicatorPosition: "end"
    }, TreeProps))));
  }))), /*#__PURE__*/React.createElement(Line, {
    gap: 1,
    direction: "row",
    align: "center",
    justify: "center",
    className: classNames([staticClassName('Timer', theme) && ['onesy-Timer-actions'], classes.actions])
  }, status === 'initial' && /*#__PURE__*/React.createElement(Fade, {
    in: true,
    add: true
  }, /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement(Tooltip, _extends({
    name: l('Start')
  }, TooltipProps), /*#__PURE__*/React.createElement(IconButton, _extends({
    onClick: onStart
  }, IconButtonProps), /*#__PURE__*/React.createElement(IconStart, null))))), status === 'running' && /*#__PURE__*/React.createElement(Fade, {
    in: true,
    add: true
  }, /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement(Tooltip, _extends({
    name: l('Flag')
  }, TooltipProps), /*#__PURE__*/React.createElement(IconButton, _extends({
    onClick: onFlag
  }, IconButtonProps), /*#__PURE__*/React.createElement(IconFlag, null))), /*#__PURE__*/React.createElement(Tooltip, _extends({
    name: l('Pause')
  }, TooltipProps), /*#__PURE__*/React.createElement(IconButton, _extends({
    onClick: onPause
  }, IconButtonProps), /*#__PURE__*/React.createElement(IconPause, null))))), status === 'paused' && /*#__PURE__*/React.createElement(Fade, {
    in: true,
    add: true
  }, /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement(Tooltip, _extends({
    name: l('Stop')
  }, TooltipProps), /*#__PURE__*/React.createElement(IconButton, _extends({
    onClick: onStop
  }, IconButtonProps), /*#__PURE__*/React.createElement(IconStop, null))), /*#__PURE__*/React.createElement(Tooltip, _extends({
    name: l('Resume')
  }, TooltipProps), /*#__PURE__*/React.createElement(IconButton, _extends({
    onClick: onResume
  }, IconButtonProps), /*#__PURE__*/React.createElement(IconStart, null)))))));
});
Timer.displayName = 'onesy-Timer';
export default Timer;